@@ -1,5 +1,6 @@
 /* GtkToolkit.java -- Implements an AWT Toolkit using GTK for peers
-   Copyright (C) 1998, 1999, 2002, 2003, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2002, 2003, 2004, 2005, 2006
+   Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -88,9 +89,9 @@ public class GtkToolkit extends gnu.java.awt.ClasspathToolkit
 {
   Hashtable containers = new Hashtable();
   static EventQueue q;
-  static Clipboard systemClipboard;
   static boolean useGraphics2dSet;
   static boolean useGraphics2d;
+  static Thread mainThread;
 
   public static boolean useGraphics2D()
   {
@@ -121,11 +122,19 @@ public class GtkToolkit extends gnu.java.awt.ClasspathToolkit
       portableNativeSync = 0;   // false
 
     gtkInit(portableNativeSync);
+
+    mainThread = new Thread ("GTK main thread")
+      {
+        public void run ()
+        {
+          gtkMain ();
+        }
+      };
+    mainThread.start ();
   }
 
   public GtkToolkit ()
   {
-    systemClipboard = new GtkClipboard ();
   }
 
   public native void beep();
@@ -151,140 +160,93 @@ public class GtkToolkit extends gnu.java.awt.ClasspathToolkit
   }
 
   /** 
-   * A helper class to return to clients in cases where a BufferedImage is
-   * desired but its construction fails.
+   * Helper to return either a Image -- the argument -- or a
+   * GtkImage with the errorLoading flag set if the argument is null.
    */
-  private class GtkErrorImage extends Image
-  {
-    public GtkErrorImage()
-    {
-    }
-
-    public int getWidth(ImageObserver observer)
-    {
-      return -1;
-    }
-
-    public int getHeight(ImageObserver observer)
-    {
-      return -1;
-    }
-
-    public ImageProducer getSource()
-    {
-
-      return new ImageProducer() 
-        {          
-          HashSet consumers = new HashSet();          
-          public void addConsumer(ImageConsumer ic)
-          {
-            consumers.add(ic);
-          }
-
-          public boolean isConsumer(ImageConsumer ic)
-          {
-            return consumers.contains(ic);
-          }
-
-          public void removeConsumer(ImageConsumer ic)
-          {
-            consumers.remove(ic);
-          }
-
-          public void startProduction(ImageConsumer ic)
-          {
-            consumers.add(ic);
-            Iterator i = consumers.iterator();
-            while(i.hasNext())
-              {
-                ImageConsumer c = (ImageConsumer) i.next();
-                c.imageComplete(ImageConsumer.IMAGEERROR);
-              }
-          }
-          public void requestTopDownLeftRightResend(ImageConsumer ic)
-          {
-            startProduction(ic);
-          }        
-        };
-    }
-
-    public Graphics getGraphics() 
-    { 
-      return null; 
-    }
-
-    public Object getProperty(String name, ImageObserver observer)
-    {
-      return null;
-    }
-    public Image getScaledInstance(int width, int height, int flags)
-    {
-      return new GtkErrorImage();
-    }
-
-    public void flush() 
-    {
-    }
-  }
-
-
-  /** 
-   * Helper to return either a BufferedImage -- the argument -- or a
-   * GtkErrorImage if the argument is null.
-   */
-
-  private Image bufferedImageOrError(BufferedImage b)
+  private Image imageOrError(Image b)
   {
     if (b == null) 
-      return new GtkErrorImage();
+      return GtkImage.getErrorImage();
     else
       return b;
   }
 
-
   public Image createImage (String filename)
   {
-    if (useGraphics2D())
-      return bufferedImageOrError(GdkPixbufDecoder.createBufferedImage (filename));
-    else
-      return new GtkImage (filename);
+    if (filename.length() == 0)
+      return new GtkImage ();
+    
+    Image image;
+    try
+      {
+	if (useGraphics2D())
+	  image = GdkPixbufDecoder.createBufferedImage(filename);
+	else
+	  image = new GtkImage(filename);
+      }
+    catch (IllegalArgumentException iae)
+      {
+	image = null;
+      }
+    return imageOrError(image);
   }
 
   public Image createImage (URL url)
   {
-    if (useGraphics2D())
-      return bufferedImageOrError(GdkPixbufDecoder.createBufferedImage (url));
-    else
+    Image image;
+    try
       {
-        GdkPixbufDecoder d = new GdkPixbufDecoder (url);
-        GtkImage image = new GtkImage (d);
-        return image;        
+	if (useGraphics2D())
+	  image = GdkPixbufDecoder.createBufferedImage(url);
+	else
+	  image = new GtkImage(url);
       }
+    catch (IllegalArgumentException iae)
+      {
+	image = null;
+      }
+    return imageOrError(image);
   }
 
   public Image createImage (ImageProducer producer) 
   {
-    if (useGraphics2D())
-      return bufferedImageOrError(GdkPixbufDecoder.createBufferedImage (producer));
-    else
-      return new GtkImage (producer);
+    Image image;
+    try
+      {
+	if (useGraphics2D())
+	  image = GdkPixbufDecoder.createBufferedImage(producer);
+	else
+	  image = new GtkImage(producer);
+      }
+    catch (IllegalArgumentException iae)
+      {
+	image = null;
+      }
+    return imageOrError(image);
   }
 
   public Image createImage (byte[] imagedata, int imageoffset,
 			    int imagelength)
   {
-    if (useGraphics2D())
-      return bufferedImageOrError(GdkPixbufDecoder.createBufferedImage (imagedata,
-                                                   imageoffset, 
-                                                                        imagelength));
-    else
+    Image image;
+    try
       {
-        GdkPixbufDecoder d = new GdkPixbufDecoder (imagedata,
-                                                   imageoffset, 
-                                                   imagelength);
-        GtkImage image = new GtkImage (d);
-        return image;        
+	if (useGraphics2D())
+	  image = GdkPixbufDecoder.createBufferedImage(imagedata,
+						       imageoffset, 
+						       imagelength);
+	else
+	  {
+	    byte[] datacopy = new byte[imagelength];
+	    System.arraycopy(imagedata, imageoffset, datacopy, 0, imagelength);
+	    return new GtkImage(datacopy);
+	  }
       }
+    catch (IllegalArgumentException iae)
+      {
+	image = null;
+      }
+    return imageOrError(image);
   }
   
   /**
@@ -395,7 +357,11 @@ public class GtkToolkit extends gnu.java.awt.ClasspathToolkit
 
   public Clipboard getSystemClipboard() 
   {
-    return systemClipboard;
+    SecurityManager secman = System.getSecurityManager();
+    if (secman != null)
+      secman.checkSystemClipboardAccess();
+
+    return GtkClipboard.getInstance();
   }
 
   /**
@@ -599,7 +565,6 @@ public class GtkToolkit extends gnu.java.awt.ClasspathToolkit
         if (q == null)
           {
             q = new EventQueue();
-            GtkGenericPeer.enableQueue (q);
           }
       }    
     return q;
@@ -628,7 +593,7 @@ public class GtkToolkit extends gnu.java.awt.ClasspathToolkit
 
   public GraphicsEnvironment getLocalGraphicsEnvironment()
   {
-    return new GdkGraphicsEnvironment(this);
+    return new GdkGraphicsEnvironment();
   }
 
   public Font createFont(int format, InputStream stream)
@@ -646,8 +611,5 @@ public class GtkToolkit extends gnu.java.awt.ClasspathToolkit
     GdkPixbufDecoder.registerSpis(reg);
   }
 
-  public native boolean nativeQueueEmpty();
-  public native void wakeNativeQueue();  
-  public native void iterateNativeQueue(EventQueue locked, boolean block);
-
+  public static native void gtkMain();
 } // class GtkToolkit
