@@ -281,50 +281,53 @@ public class Window extends Container implements Accessible
   public void show()
   {
     synchronized (getTreeLock())
-    {
-    if (parent != null && !parent.isDisplayable())
-      parent.addNotify();
-    if (peer == null)
-      addNotify();
-
-    // Show visible owned windows.
-	Iterator e = ownedWindows.iterator();
-	while(e.hasNext())
-	  {
-	    Window w = (Window)(((Reference) e.next()).get());
-	    if (w != null)
-	      {
-		if (w.isVisible())
-		  w.getPeer().setVisible(true);
-	      }
-     	    else
-	      // Remove null weak reference from ownedWindows.
-	      // Unfortunately this can't be done in the Window's
-	      // finalize method because there is no way to guarantee
-	      // synchronous access to ownedWindows there.
-	      e.remove();
-	  }
-    validate();
-    super.show();
-    toFront();
-
-    KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager ();
-    manager.setGlobalFocusedWindow (this);
-    
-    if (!shown)
       {
-        FocusTraversalPolicy policy = getFocusTraversalPolicy ();
-        Component initialFocusOwner = null;
+        if (parent != null && ! parent.isDisplayable())
+          parent.addNotify();
+        if (peer == null)
+          addNotify();
 
-        if (policy != null)
-          initialFocusOwner = policy.getInitialComponent (this);
+        validate();
+        if (visible)
+          toFront();
+        else
+          {
+            super.show();
+            // Show visible owned windows.
+            Iterator e = ownedWindows.iterator();
+            while (e.hasNext())
+              {
+                Window w = (Window) (((Reference) e.next()).get());
+                if (w != null)
+                  {
+                    if (w.isVisible())
+                      w.getPeer().setVisible(true);
+                  }
+                else
+                  // Remove null weak reference from ownedWindows.
+                  // Unfortunately this can't be done in the Window's
+                  // finalize method because there is no way to guarantee
+                  // synchronous access to ownedWindows there.
+                  e.remove();
+              }
+          }
+        KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
+        manager.setGlobalFocusedWindow(this);
 
-        if (initialFocusOwner != null)
-          initialFocusOwner.requestFocusInWindow ();
+        if (! shown)
+          {
+            FocusTraversalPolicy policy = getFocusTraversalPolicy();
+            Component initialFocusOwner = null;
 
-        shown = true;
+            if (policy != null)
+              initialFocusOwner = policy.getInitialComponent(this);
+
+            if (initialFocusOwner != null)
+              initialFocusOwner.requestFocusInWindow();
+
+            shown = true;
+          }
       }
-    }
   }
 
   public void hide()
@@ -803,20 +806,81 @@ public class Window extends Container implements Accessible
     return isVisible();
   }
 
-  public void setLocationRelativeTo (Component c)
+  public void setLocationRelativeTo(Component c)
   {
-    if (c == null || !c.isShowing ())
+    int x = 0;
+    int y = 0;
+    
+    if (c == null || !c.isShowing())
       {
-        int x = 0;
-        int y = 0;
-
-        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment ();
-        Point center = ge.getCenterPoint ();
+        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
+        Point center = ge.getCenterPoint();
         x = center.x - (width / 2);
         y = center.y - (height / 2);
-        setLocation (x, y);
       }
-    // FIXME: handle case where component is non-null.
+    else
+      {
+        int cWidth = c.getWidth();
+        int cHeight = c.getHeight();
+        Dimension screenSize = getToolkit().getScreenSize();
+
+        x = c.getLocationOnScreen().x;
+        y = c.getLocationOnScreen().y;
+        
+        // If bottom of component is cut off, window placed
+        // on the left or the right side of component
+        if ((y + cHeight) > screenSize.height)
+          {
+            // If the right side of the component is closer to the center
+            if ((screenSize.width / 2 - x) <= 0)
+              {
+                if ((x - width) >= 0)
+                  x -= width;
+                else
+                  x = 0;
+              }
+            else
+              {
+                if ((x + cWidth + width) <= screenSize.width)
+                  x += cWidth;
+                else
+                  x = screenSize.width - width;
+              }
+
+            y = screenSize.height - height;
+          }
+        else if (cWidth > width || cHeight > height)
+          {
+            // If right side of component is cut off
+            if ((x + width) > screenSize.width)
+              x = screenSize.width - width;
+            // If left side of component is cut off
+            else if (x < 0)
+              x = 0;
+            else
+              x += (cWidth - width) / 2;
+            
+            y += (cHeight - height) / 2;
+          }
+        else
+          {
+            // If right side of component is cut off
+            if ((x + width) > screenSize.width)
+              x = screenSize.width - width;
+            // If left side of component is cut off
+            else if (x < 0 || (x - (width - cWidth) / 2) < 0)
+              x = 0;
+            else
+              x -= (width - cWidth) / 2;
+
+            if ((y - (height - cHeight) / 2) > 0)
+              y -= (height - cHeight) / 2;
+            else
+              y = 0;
+          }
+      }
+
+    setLocation(x, y);
   }
 
   /**
