@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -38,6 +38,7 @@ exception statement from your version. */
 package java.security;
 
 import gnu.java.security.Engine;
+import gnu.java.security.jce.prng.Sha160RandomSpi;
 
 import java.util.Enumeration;
 import java.util.Random;
@@ -126,7 +127,7 @@ public class SecureRandom extends Random
       }
 
     // Nothing found. Fall back to SHA1PRNG
-    secureRandomSpi = new gnu.java.security.provider.SHA1PRNG();
+    secureRandomSpi = new Sha160RandomSpi();
   }
 
   /**
