@@ -1,5 +1,5 @@
 /* BitSet.java -- A vector of bits.
-   Copyright (C) 1998, 1999, 2000, 2001, 2004  Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -61,8 +61,8 @@ import java.io.Serializable;
  * undefined.
  *
  * @author Jochen Hoenicke
- * @author Tom Tromey <tromey@cygnus.com>
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Tom Tromey (tromey@cygnus.com)
+ * @author Eric Blake (ebb9@email.byu.edu)
  * @status updated to 1.4
  */
 public class BitSet implements Cloneable, Serializable
@@ -116,8 +116,8 @@ public class BitSet implements Cloneable, Serializable
    * given <code>set</code>.  This means it builds the intersection
    * of the two sets.  The result is stored into this bit set.
    *
-   * @param set the second bit set
-   * @throws NullPointerException if set is null
+   * @param bs the second bit set
+   * @throws NullPointerException if bs is null
    */
   public void and(BitSet bs)
   {
@@ -131,13 +131,13 @@ public class BitSet implements Cloneable, Serializable
 
   /**
    * Performs the logical AND operation on this bit set and the
-   * complement of the given <code>set</code>.  This means it
+   * complement of the given <code>bs</code>.  This means it
    * selects every element in the first set, that isn't in the
    * second set.  The result is stored into this bit set and is
    * effectively the set difference of the two.
    *
-   * @param set the second bit set
-   * @throws NullPointerException if set is null
+   * @param bs the second bit set
+   * @throws NullPointerException if bs is null
    * @since 1.2
    */
   public void andNot(BitSet bs)
@@ -190,12 +190,12 @@ public class BitSet implements Cloneable, Serializable
   }
 
   /**
-   * Removes the integer <code>bitIndex</code> from this set. That is
+   * Removes the integer <code>pos</code> from this set. That is
    * the corresponding bit is cleared.  If the index is not in the set,
    * this method does nothing.
    *
-   * @param bitIndex a non-negative integer
-   * @throws IndexOutOfBoundsException if bitIndex &lt; 0
+   * @param pos a non-negative integer
+   * @throws IndexOutOfBoundsException if pos &lt; 0
    */
   public void clear(int pos)
   {
@@ -336,8 +336,8 @@ public class BitSet implements Cloneable, Serializable
    * set, otherwise false.
    *
    * @param pos a non-negative integer
-   * @return the value of the bit at the specified index
-   * @throws IndexOutOfBoundsException if the index is negative
+   * @return the value of the bit at the specified position
+   * @throws IndexOutOfBoundsException if the pos is negative
    */
   public boolean get(int pos)
   {
@@ -380,7 +380,7 @@ public class BitSet implements Cloneable, Serializable
       }
 
     int len = Math.min(hi_offset, bits.length - 1);
-    int reverse = ~lo_bit;
+    int reverse = 64 - lo_bit;
     int i;
     for (i = 0; lo_offset < len; lo_offset++, i++)
       bs.bits[i] = ((bits[lo_offset] >>> lo_bit)
